(window["webpackJsonp"]=window["webpackJsonp"]||[]).push([["timings"],{"1a3c":function(t,n,e){"use strict";e.r(n);var s=function(){var t=this,n=t.$createElement,e=t._self._c||n;return e("div",{staticClass:"about"},[e("pc-socket-test",{on:{pipeline:t.on_socket_event,"did-connect":t.run_timings}}),e("h3",[t._v("Summary Timings:")]),t.timings?t._l(t.timings.results,function(n){return e("div",{key:n.name,staticClass:"timing mb-5"},[e("div",{staticClass:"tl-name"},[t._v(t._s(n.timeline_name))]),t._l(n.intervals,function(n){return e("div",{key:n.name,staticClass:"event"},[t._v("\n          "+t._s(n.name)+" - Duration - "+t._s(n.duration)+" seconds\n          "),e("h5",{staticClass:"title"},[t._v("\n          Events:\n          ")]),t._l(n.events,function(n){return e("div",{key:n.name,staticClass:"evts ml-5"},[t._v("\n            "+t._s(n.name)+" - "+t._s(n.timestamp)+"\n\n          ")])}),n.intervals&&n.intervals.length>0?e("div",{staticClass:"nested-intervals ml-5"},[e("h5",{staticClass:"mt-4"},[t._v("Nested Intervals")]),t._l(n.intervals,function(n,s){return e("div",{key:s,staticClass:"nester-interval__list"},t._l(n,function(n,s){return e("div",{key:s,staticClass:"nester-interval__single mt-3"},[t._v("\n                "+t._s(n.name)+" - Duration - "+t._s(n.duration)+" seconds\n                "),e("h6",{staticClass:"title"},[t._v("\n                  Events:\n                ")]),t._l(n.events,function(n){return e("div",{key:n.name,staticClass:"evts ml-5"},[t._v("\n                  "+t._s(n.name)+" - "+t._s(n.timestamp)+"\n\n                ")])})],2)}),0)})],2):t._e()],2)})],2)}):t._e()],2)},i=[],a=(e("28a5"),e("cebc")),c=(e("96cf"),e("3b8d")),r=e("97c5"),o=(e("db49"),e("bc3a"),e("2f62")),u={name:"home",data:function(){return{timings:null}},components:{"pc-socket-test":r["a"]},mounted:function(){var t=Object(c["a"])(regeneratorRuntime.mark(function t(){return regeneratorRuntime.wrap(function(t){while(1)switch(t.prev=t.next){case 0:this.websocket&&this.run_timings();case 1:case"end":return t.stop()}},t,this)}));function n(){return t.apply(this,arguments)}return n}(),methods:{run_timings:function(){this.websocket.emit("my event",{action:"timings"})},on_socket_event:function(t){console.log("pipe",t),"execution_history_did_become_available"===t.event_type&&(this.timings=t)}},computed:Object(a["a"])({},Object(o["e"])(["websocket"]),{envs:function(){if(!this.diagnostics)return[];var t=[],n=this.diagnostics.env;for(var e in n){var s=n[e];e.toLowerCase().indexOf("path")>=0&&(s=s.split(":")),t.push({name:e,value:s})}return t}})},l=u,v=(e("bcf6"),e("2877")),m=Object(v["a"])(l,s,i,!1,null,"6006c62a",null);n["default"]=m.exports},"7c9b":function(t,n,e){},bcf6:function(t,n,e){"use strict";var s=e("7c9b"),i=e.n(s);i.a}}]);
//# sourceMappingURL=timings.51f005d6.js.map