(window["webpackJsonp"]=window["webpackJsonp"]||[]).push([["more"],{"16a5":function(t,e,n){"use strict";var s=n("812e"),i=n.n(s);i.a},"5dde":function(t,e,n){"use strict";n.r(e);var s=function(){var t=this,e=t.$createElement,n=t._self._c||e;return n("div",{staticClass:"about"},[n("pc-socket-test",{on:{pipeline:t.on_socket_event,"did-connect":t.run_diagnostics}}),n("h3",[t._v("Summary:")]),t.diagnostics?[n("div",{staticClass:"mb-5"},[t._v("Engine: "),n("span",{staticClass:"badge-light"},[t._v(t._s(t.diagnostics.engine))])]),n("h5",[t._v("Environment:")]),n("div",{staticClass:"envs"},t._l(t.envs,function(e){return n("div",{key:e.name},[n("div",{staticClass:"envs__single env"},[n("div",{staticClass:"env__name"},[t._v("\n            "+t._s(e.name)+"\n          ")]),n("div",{staticClass:"env__value"},[t._v("\n            "+t._s(e.value)+"\n          ")])])])}),0),n("h5",{staticClass:"mt-5"},[t._v("Modules ("+t._s(t.diagnostics.modules.length)+" total):")]),n("div",{staticClass:"modules"},t._l(t.diagnostics.modules,function(e){return n("div",{key:e,staticClass:"modules__single"},[n("div",{},[t._v(t._s(e))])])}),0)]:t._e()],2)},i=[],a=(n("28a5"),n("cebc")),c=(n("96cf"),n("3b8d")),o=n("97c5"),r=(n("db49"),n("bc3a"),n("2f62")),u={name:"home",data:function(){return{diagnostics:null}},components:{"pc-socket-test":o["a"]},mounted:function(){var t=Object(c["a"])(regeneratorRuntime.mark(function t(){return regeneratorRuntime.wrap(function(t){while(1)switch(t.prev=t.next){case 0:this.websocket&&this.run_diagnostics();case 1:case"end":return t.stop()}},t,this)}));function e(){return t.apply(this,arguments)}return e}(),methods:{run_diagnostics:function(){this.websocket.emit("my event",{action:"diagnostics"})},test_run_completed:function(t){this.test_run=t.coverage},on_socket_event:function(t){console.log("pipe",t),"diagnostics_did_become_available"===t.event_type&&(this.diagnostics=t)}},computed:Object(a["a"])({},Object(r["e"])(["websocket"]),{envs:function(){if(!this.diagnostics)return[];var t=[],e=this.diagnostics.env;for(var n in e){var s=e[n];n.toLowerCase().indexOf("path")>=0&&(s=s.split(":")),t.push({name:n,value:s})}return t}})},d=u,v=(n("16a5"),n("2877")),l=Object(v["a"])(d,s,i,!1,null,"5604f76a",null);e["default"]=l.exports},"812e":function(t,e,n){}}]);
//# sourceMappingURL=more.0d5111d8.js.map