.env[data-v-5604f76a]{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-direction:row;flex-direction:row}.env__name[data-v-5604f76a]{-webkit-box-flex:1;-ms-flex:1 0 25%;flex:1 0 25%}.env__value[data-v-5604f76a]{-webkit-box-flex:1;-ms-flex:1 0 75%;flex:1 0 75%}.modules[data-v-5604f76a]{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-direction:row;flex-direction:row;-ms-flex-wrap:wrap;flex-wrap:wrap}.modules__single[data-v-5604f76a]{width:500px}